Class: Xlsxrb::StreamWriter::WorksheetProxy
- Inherits:
-
Object
- Object
- Xlsxrb::StreamWriter::WorksheetProxy
- Defined in:
- lib/xlsxrb.rb,
sig/generated/xlsxrb.rbs
Overview
Proxy object yielded by the sheet method to prevent writing to inactive sheets.
Instance Method Summary collapse
-
#app_property ⇒ void
Delegates to StreamWriter#app_property.
-
#auto_filter ⇒ void
Delegates to StreamWriter#auto_filter.
-
#chart ⇒ void
Delegates to StreamWriter#chart.
-
#cleanup! ⇒ void
Delegates to StreamWriter#cleanup!.
-
#column ⇒ void
Delegates to StreamWriter#column.
-
#comment ⇒ void
Delegates to StreamWriter#comment.
-
#conditional_format ⇒ void
Delegates to StreamWriter#conditional_format.
-
#core_property ⇒ void
Delegates to StreamWriter#core_property.
-
#custom_property ⇒ void
Delegates to StreamWriter#custom_property.
-
#defined_name ⇒ void
Delegates to StreamWriter#defined_name.
-
#filter_column ⇒ void
Delegates to StreamWriter#filter_column.
-
#freeze_pane ⇒ void
Delegates to StreamWriter#freeze_pane.
-
#header_footer ⇒ void
Delegates to StreamWriter#header_footer.
-
#hyperlink ⇒ void
Delegates to StreamWriter#hyperlink.
-
#image ⇒ void
Delegates to StreamWriter#image.
-
#initialize(writer, sheet_name) ⇒ WorksheetProxy
constructor
A new instance of WorksheetProxy.
-
#internal_sheet_setup ⇒ void
Delegates to StreamWriter#internal_sheet_setup.
-
#merge ⇒ void
Delegates to StreamWriter#merge.
-
#page_break_col ⇒ void
Delegates to StreamWriter#page_break_col.
-
#page_break_row ⇒ void
Delegates to StreamWriter#page_break_row.
-
#page_margins ⇒ void
Delegates to StreamWriter#page_margins.
-
#page_setup ⇒ void
Delegates to StreamWriter#page_setup.
-
#pivot_table ⇒ void
Delegates to StreamWriter#pivot_table.
-
#print_area ⇒ void
Delegates to StreamWriter#print_area.
-
#print_options ⇒ void
Delegates to StreamWriter#print_options.
-
#print_titles ⇒ void
Delegates to StreamWriter#print_titles.
-
#properties ⇒ void
Delegates to StreamWriter#properties.
-
#protect_sheet ⇒ void
Delegates to StreamWriter#protect_sheet.
-
#protect_workbook ⇒ void
Delegates to StreamWriter#protect_workbook.
-
#row ⇒ void
Delegates to StreamWriter#row.
-
#select_cell ⇒ void
Delegates to StreamWriter#select_cell.
-
#shape ⇒ void
Delegates to StreamWriter#shape.
-
#sheet_properties ⇒ void
Delegates to StreamWriter#sheet_properties.
-
#sheet_view ⇒ void
Delegates to StreamWriter#sheet_view.
-
#sort_state ⇒ void
Delegates to StreamWriter#sort_state.
-
#sparkline_group ⇒ void
Delegates to StreamWriter#sparkline_group.
-
#split_pane ⇒ void
Delegates to StreamWriter#split_pane.
-
#style ⇒ Object
Delegates to StreamWriter#style.
-
#table ⇒ void
Delegates to StreamWriter#table.
-
#validate_data ⇒ void
Delegates to StreamWriter#validate_data.
-
#workbook_property ⇒ void
Delegates to StreamWriter#workbook_property.
Constructor Details
#initialize(writer, sheet_name) ⇒ WorksheetProxy
Returns a new instance of WorksheetProxy.
1613 1614 1615 1616 |
# File 'lib/xlsxrb.rb', line 1613 def initialize(writer, sheet_name) @writer = writer @sheet_name = sheet_name end |
Instance Method Details
#app_property ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#app_property. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1974 1975 1976 1977 1978 1979 1980 1981 |
# File 'lib/xlsxrb.rb', line 1974 def app_property(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.app_property(...) # simplecov:enable end |
#auto_filter ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#auto_filter. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1706 1707 1708 1709 1710 |
# File 'lib/xlsxrb.rb', line 1706 def auto_filter(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.auto_filter(...) end |
#chart ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#chart. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1686 1687 1688 1689 1690 |
# File 'lib/xlsxrb.rb', line 1686 def chart(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.chart(...) end |
#cleanup! ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#cleanup!. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1766 1767 1768 1769 1770 1771 1772 1773 |
# File 'lib/xlsxrb.rb', line 1766 def cleanup!(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name # simplecov:disable # Edge case / untested delegation block @writer.cleanup!(...) # simplecov:enable end |
#column ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#column. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1676 1677 1678 1679 1680 |
# File 'lib/xlsxrb.rb', line 1676 def column(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.column(...) end |
#comment ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#comment. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1779 1780 1781 1782 1783 |
# File 'lib/xlsxrb.rb', line 1779 def comment(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.comment(...) end |
#conditional_format ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#conditional_format. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1746 1747 1748 1749 1750 |
# File 'lib/xlsxrb.rb', line 1746 def conditional_format(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.conditional_format(...) end |
#core_property ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#core_property. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1898 1899 1900 1901 1902 1903 1904 1905 |
# File 'lib/xlsxrb.rb', line 1898 def core_property(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.core_property(...) # simplecov:enable end |
#custom_property ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#custom_property. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1997 1998 1999 2000 2001 2002 2003 2004 |
# File 'lib/xlsxrb.rb', line 1997 def custom_property(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.custom_property(...) # simplecov:enable end |
#defined_name ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#defined_name. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1829 1830 1831 1832 1833 |
# File 'lib/xlsxrb.rb', line 1829 def defined_name(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.defined_name(...) end |
#filter_column ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#filter_column. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1716 1717 1718 1719 1720 |
# File 'lib/xlsxrb.rb', line 1716 def filter_column(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.filter_column(...) end |
#freeze_pane ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#freeze_pane. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1839 1840 1841 1842 1843 |
# File 'lib/xlsxrb.rb', line 1839 def freeze_pane(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.freeze_pane(...) end |
#header_footer ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#header_footer. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1941 1942 1943 1944 1945 |
# File 'lib/xlsxrb.rb', line 1941 def (...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.(...) end |
#hyperlink ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#hyperlink. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1696 1697 1698 1699 1700 |
# File 'lib/xlsxrb.rb', line 1696 def hyperlink(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.hyperlink(...) end |
#image ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#image. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
2010 2011 2012 2013 2014 |
# File 'lib/xlsxrb.rb', line 2010 def image(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.image(...) end |
#internal_sheet_setup ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#internal_sheet_setup. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1653 1654 1655 1656 1657 1658 1659 1660 |
# File 'lib/xlsxrb.rb', line 1653 def internal_sheet_setup(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name # simplecov:disable # Edge case / untested delegation block @writer.internal_sheet_setup(...) # simplecov:enable end |
#merge ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#merge. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1633 1634 1635 1636 1637 |
# File 'lib/xlsxrb.rb', line 1633 def merge(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.merge(...) end |
#page_break_col ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#page_break_col. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
2043 2044 2045 2046 2047 2048 2049 2050 |
# File 'lib/xlsxrb.rb', line 2043 def page_break_col(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_break_col(...) # simplecov:enable end |
#page_break_row ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#page_break_row. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
2030 2031 2032 2033 2034 2035 2036 2037 |
# File 'lib/xlsxrb.rb', line 2030 def page_break_row(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_break_row(...) # simplecov:enable end |
#page_margins ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#page_margins. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1921 1922 1923 1924 1925 |
# File 'lib/xlsxrb.rb', line 1921 def page_margins(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_margins(...) end |
#page_setup ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#page_setup. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1931 1932 1933 1934 1935 |
# File 'lib/xlsxrb.rb', line 1931 def page_setup(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_setup(...) end |
#pivot_table ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#pivot_table. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1789 1790 1791 1792 1793 |
# File 'lib/xlsxrb.rb', line 1789 def pivot_table(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.pivot_table(...) end |
#print_area ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#print_area. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1849 1850 1851 1852 1853 1854 1855 1856 |
# File 'lib/xlsxrb.rb', line 1849 def print_area(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.print_area(...) # simplecov:enable end |
#print_options ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#print_options. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1951 1952 1953 1954 1955 |
# File 'lib/xlsxrb.rb', line 1951 def (...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.(...) end |
#print_titles ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#print_titles. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1862 1863 1864 1865 1866 1867 1868 1869 |
# File 'lib/xlsxrb.rb', line 1862 def print_titles(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.print_titles(...) # simplecov:enable end |
#properties ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#properties. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1961 1962 1963 1964 1965 1966 1967 1968 |
# File 'lib/xlsxrb.rb', line 1961 def properties(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.properties(...) # simplecov:enable end |
#protect_sheet ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#protect_sheet. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1987 1988 1989 1990 1991 |
# File 'lib/xlsxrb.rb', line 1987 def protect_sheet(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.protect_sheet(...) end |
#protect_workbook ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#protect_workbook. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1885 1886 1887 1888 1889 1890 1891 1892 |
# File 'lib/xlsxrb.rb', line 1885 def protect_workbook(...) # simplecov:disable # Edge case / untested delegation block raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.protect_workbook(...) # simplecov:enable end |
#row ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#row. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1666 1667 1668 1669 1670 |
# File 'lib/xlsxrb.rb', line 1666 def row(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.row(...) end |
#select_cell ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#select_cell. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1911 1912 1913 1914 1915 |
# File 'lib/xlsxrb.rb', line 1911 def select_cell(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.select_cell(...) end |
#shape ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#shape. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1643 1644 1645 1646 1647 |
# File 'lib/xlsxrb.rb', line 1643 def shape(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.shape(...) end |
#sheet_properties ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#sheet_properties. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1819 1820 1821 1822 1823 |
# File 'lib/xlsxrb.rb', line 1819 def sheet_properties(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sheet_properties(...) end |
#sheet_view ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#sheet_view. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
2020 2021 2022 2023 2024 |
# File 'lib/xlsxrb.rb', line 2020 def sheet_view(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sheet_view(...) end |
#sort_state ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#sort_state. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1726 1727 1728 1729 1730 |
# File 'lib/xlsxrb.rb', line 1726 def sort_state(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sort_state(...) end |
#sparkline_group ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#sparkline_group. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1799 1800 1801 1802 1803 |
# File 'lib/xlsxrb.rb', line 1799 def sparkline_group(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sparkline_group(...) end |
#split_pane ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#split_pane. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1875 1876 1877 1878 1879 |
# File 'lib/xlsxrb.rb', line 1875 def split_pane(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.split_pane(...) end |
#style(args, kwargs) ⇒ void #style(args, kwargs) ⇒ void
Delegates to StreamWriter#style. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped : (*untyped args, **untyped kwargs) ?{ (Xlsxrb::StyleBuilder) -> void } -> untyped
1623 1624 1625 1626 1627 |
# File 'lib/xlsxrb.rb', line 1623 def style(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.style(...) end |
#table ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#table. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1756 1757 1758 1759 1760 |
# File 'lib/xlsxrb.rb', line 1756 def table(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.table(...) end |
#validate_data ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#validate_data. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1736 1737 1738 1739 1740 |
# File 'lib/xlsxrb.rb', line 1736 def validate_data(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.validate_data(...) end |
#workbook_property ⇒ void
This method returns an undefined value.
Delegates to StreamWriter#workbook_property. : (*untyped args, **untyped kwargs) ?{ (*untyped) -> untyped } -> untyped
1809 1810 1811 1812 1813 |
# File 'lib/xlsxrb.rb', line 1809 def workbook_property(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.workbook_property(...) end |