Class: Xlsxrb::StreamWriter::WorksheetProxy
- Inherits:
-
Object
- Object
- Xlsxrb::StreamWriter::WorksheetProxy
- Defined in:
- lib/xlsxrb.rb
Overview
Proxy object yielded by the sheet method to prevent writing to inactive sheets.
Instance Method Summary collapse
-
#app_property ⇒ Object
Delegates to StreamWriter#app_property.
-
#auto_filter ⇒ Object
Delegates to StreamWriter#auto_filter.
-
#chart ⇒ Object
Delegates to StreamWriter#chart.
-
#cleanup! ⇒ Object
Delegates to StreamWriter#cleanup!.
-
#column ⇒ Object
Delegates to StreamWriter#column.
-
#comment ⇒ Object
Delegates to StreamWriter#comment.
-
#conditional_format ⇒ Object
Delegates to StreamWriter#conditional_format.
-
#core_property ⇒ Object
Delegates to StreamWriter#core_property.
-
#custom_property ⇒ Object
Delegates to StreamWriter#custom_property.
-
#defined_name ⇒ Object
Delegates to StreamWriter#defined_name.
-
#filter_column ⇒ Object
Delegates to StreamWriter#filter_column.
-
#freeze_pane ⇒ Object
Delegates to StreamWriter#freeze_pane.
-
#header_footer ⇒ Object
Delegates to StreamWriter#header_footer.
-
#hyperlink ⇒ Object
Delegates to StreamWriter#hyperlink.
-
#image ⇒ Object
Delegates to StreamWriter#image.
-
#initialize(writer, sheet_name) ⇒ WorksheetProxy
constructor
A new instance of WorksheetProxy.
-
#internal_sheet_setup ⇒ Object
Delegates to StreamWriter#internal_sheet_setup.
-
#merge ⇒ Object
Delegates to StreamWriter#merge.
-
#page_break_col ⇒ Object
Delegates to StreamWriter#page_break_col.
-
#page_break_row ⇒ Object
Delegates to StreamWriter#page_break_row.
-
#page_margins ⇒ Object
Delegates to StreamWriter#page_margins.
-
#page_setup ⇒ Object
Delegates to StreamWriter#page_setup.
-
#pivot_table ⇒ Object
Delegates to StreamWriter#pivot_table.
-
#print_area ⇒ Object
Delegates to StreamWriter#print_area.
-
#print_options ⇒ Object
Delegates to StreamWriter#print_options.
-
#print_titles ⇒ Object
Delegates to StreamWriter#print_titles.
-
#properties ⇒ Object
Delegates to StreamWriter#properties.
-
#protect_sheet ⇒ Object
Delegates to StreamWriter#protect_sheet.
-
#protect_workbook ⇒ Object
Delegates to StreamWriter#protect_workbook.
-
#row ⇒ Object
Delegates to StreamWriter#row.
-
#select_cell ⇒ Object
Delegates to StreamWriter#select_cell.
-
#shape ⇒ Object
Delegates to StreamWriter#shape.
-
#sheet_properties ⇒ Object
Delegates to StreamWriter#sheet_properties.
-
#sheet_view ⇒ Object
Delegates to StreamWriter#sheet_view.
-
#sort_state ⇒ Object
Delegates to StreamWriter#sort_state.
-
#sparkline_group ⇒ Object
Delegates to StreamWriter#sparkline_group.
-
#split_pane ⇒ Object
Delegates to StreamWriter#split_pane.
-
#style ⇒ Object
Delegates to StreamWriter#style.
-
#table ⇒ Object
Delegates to StreamWriter#table.
-
#validate_data ⇒ Object
Delegates to StreamWriter#validate_data.
-
#workbook_property ⇒ Object
Delegates to StreamWriter#workbook_property.
Constructor Details
#initialize(writer, sheet_name) ⇒ WorksheetProxy
Returns a new instance of WorksheetProxy.
1536 1537 1538 1539 |
# File 'lib/xlsxrb.rb', line 1536 def initialize(writer, sheet_name) @writer = writer @sheet_name = sheet_name end |
Instance Method Details
#app_property ⇒ Object
Delegates to StreamWriter#app_property. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1841 1842 1843 1844 1845 |
# File 'lib/xlsxrb.rb', line 1841 def app_property(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.app_property(...) end |
#auto_filter ⇒ Object
Delegates to StreamWriter#auto_filter. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1616 1617 1618 1619 1620 |
# File 'lib/xlsxrb.rb', line 1616 def auto_filter(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.auto_filter(...) end |
#chart ⇒ Object
Delegates to StreamWriter#chart. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1598 1599 1600 1601 1602 |
# File 'lib/xlsxrb.rb', line 1598 def chart(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.chart(...) end |
#cleanup! ⇒ Object
Delegates to StreamWriter#cleanup!. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1670 1671 1672 1673 1674 |
# File 'lib/xlsxrb.rb', line 1670 def cleanup!(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.cleanup!(...) end |
#column ⇒ Object
Delegates to StreamWriter#column. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1589 1590 1591 1592 1593 |
# File 'lib/xlsxrb.rb', line 1589 def column(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.column(...) end |
#comment ⇒ Object
Delegates to StreamWriter#comment. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1679 1680 1681 1682 1683 |
# File 'lib/xlsxrb.rb', line 1679 def comment(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.comment(...) end |
#conditional_format ⇒ Object
Delegates to StreamWriter#conditional_format. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1652 1653 1654 1655 1656 |
# File 'lib/xlsxrb.rb', line 1652 def conditional_format(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.conditional_format(...) end |
#core_property ⇒ Object
Delegates to StreamWriter#core_property. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1778 1779 1780 1781 1782 |
# File 'lib/xlsxrb.rb', line 1778 def core_property(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.core_property(...) end |
#custom_property ⇒ Object
Delegates to StreamWriter#custom_property. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1859 1860 1861 1862 1863 |
# File 'lib/xlsxrb.rb', line 1859 def custom_property(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.custom_property(...) end |
#defined_name ⇒ Object
Delegates to StreamWriter#defined_name. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1724 1725 1726 1727 1728 |
# File 'lib/xlsxrb.rb', line 1724 def defined_name(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.defined_name(...) end |
#filter_column ⇒ Object
Delegates to StreamWriter#filter_column. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1625 1626 1627 1628 1629 |
# File 'lib/xlsxrb.rb', line 1625 def filter_column(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.filter_column(...) end |
#freeze_pane ⇒ Object
Delegates to StreamWriter#freeze_pane. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1733 1734 1735 1736 1737 |
# File 'lib/xlsxrb.rb', line 1733 def freeze_pane(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.freeze_pane(...) end |
#header_footer ⇒ Object
Delegates to StreamWriter#header_footer. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1814 1815 1816 1817 1818 |
# File 'lib/xlsxrb.rb', line 1814 def (...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.(...) end |
#hyperlink ⇒ Object
Delegates to StreamWriter#hyperlink. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1607 1608 1609 1610 1611 |
# File 'lib/xlsxrb.rb', line 1607 def hyperlink(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.hyperlink(...) end |
#image ⇒ Object
Delegates to StreamWriter#image. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1868 1869 1870 1871 1872 |
# File 'lib/xlsxrb.rb', line 1868 def image(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.image(...) end |
#internal_sheet_setup ⇒ Object
Delegates to StreamWriter#internal_sheet_setup. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1571 1572 1573 1574 1575 |
# File 'lib/xlsxrb.rb', line 1571 def internal_sheet_setup(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.internal_sheet_setup(...) end |
#merge ⇒ Object
Delegates to StreamWriter#merge. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1553 1554 1555 1556 1557 |
# File 'lib/xlsxrb.rb', line 1553 def merge(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.merge(...) end |
#page_break_col ⇒ Object
Delegates to StreamWriter#page_break_col. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1895 1896 1897 1898 1899 |
# File 'lib/xlsxrb.rb', line 1895 def page_break_col(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_break_col(...) end |
#page_break_row ⇒ Object
Delegates to StreamWriter#page_break_row. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1886 1887 1888 1889 1890 |
# File 'lib/xlsxrb.rb', line 1886 def page_break_row(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_break_row(...) end |
#page_margins ⇒ Object
Delegates to StreamWriter#page_margins. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1796 1797 1798 1799 1800 |
# File 'lib/xlsxrb.rb', line 1796 def page_margins(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_margins(...) end |
#page_setup ⇒ Object
Delegates to StreamWriter#page_setup. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1805 1806 1807 1808 1809 |
# File 'lib/xlsxrb.rb', line 1805 def page_setup(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.page_setup(...) end |
#pivot_table ⇒ Object
Delegates to StreamWriter#pivot_table. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1688 1689 1690 1691 1692 |
# File 'lib/xlsxrb.rb', line 1688 def pivot_table(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.pivot_table(...) end |
#print_area ⇒ Object
Delegates to StreamWriter#print_area. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1742 1743 1744 1745 1746 |
# File 'lib/xlsxrb.rb', line 1742 def print_area(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.print_area(...) end |
#print_options ⇒ Object
Delegates to StreamWriter#print_options. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1823 1824 1825 1826 1827 |
# File 'lib/xlsxrb.rb', line 1823 def (...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.(...) end |
#print_titles ⇒ Object
Delegates to StreamWriter#print_titles. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1751 1752 1753 1754 1755 |
# File 'lib/xlsxrb.rb', line 1751 def print_titles(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.print_titles(...) end |
#properties ⇒ Object
Delegates to StreamWriter#properties. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1832 1833 1834 1835 1836 |
# File 'lib/xlsxrb.rb', line 1832 def properties(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.properties(...) end |
#protect_sheet ⇒ Object
Delegates to StreamWriter#protect_sheet. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1850 1851 1852 1853 1854 |
# File 'lib/xlsxrb.rb', line 1850 def protect_sheet(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.protect_sheet(...) end |
#protect_workbook ⇒ Object
Delegates to StreamWriter#protect_workbook. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1769 1770 1771 1772 1773 |
# File 'lib/xlsxrb.rb', line 1769 def protect_workbook(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.protect_workbook(...) end |
#row ⇒ Object
Delegates to StreamWriter#row. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1580 1581 1582 1583 1584 |
# File 'lib/xlsxrb.rb', line 1580 def row(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.row(...) end |
#select_cell ⇒ Object
Delegates to StreamWriter#select_cell. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1787 1788 1789 1790 1791 |
# File 'lib/xlsxrb.rb', line 1787 def select_cell(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.select_cell(...) end |
#shape ⇒ Object
Delegates to StreamWriter#shape. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1562 1563 1564 1565 1566 |
# File 'lib/xlsxrb.rb', line 1562 def shape(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.shape(...) end |
#sheet_properties ⇒ Object
Delegates to StreamWriter#sheet_properties. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1715 1716 1717 1718 1719 |
# File 'lib/xlsxrb.rb', line 1715 def sheet_properties(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sheet_properties(...) end |
#sheet_view ⇒ Object
Delegates to StreamWriter#sheet_view. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1877 1878 1879 1880 1881 |
# File 'lib/xlsxrb.rb', line 1877 def sheet_view(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sheet_view(...) end |
#sort_state ⇒ Object
Delegates to StreamWriter#sort_state. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1634 1635 1636 1637 1638 |
# File 'lib/xlsxrb.rb', line 1634 def sort_state(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sort_state(...) end |
#sparkline_group ⇒ Object
Delegates to StreamWriter#sparkline_group. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1697 1698 1699 1700 1701 |
# File 'lib/xlsxrb.rb', line 1697 def sparkline_group(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.sparkline_group(...) end |
#split_pane ⇒ Object
Delegates to StreamWriter#split_pane. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1760 1761 1762 1763 1764 |
# File 'lib/xlsxrb.rb', line 1760 def split_pane(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.split_pane(...) end |
#style ⇒ Object
Delegates to StreamWriter#style. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1544 1545 1546 1547 1548 |
# File 'lib/xlsxrb.rb', line 1544 def style(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.style(...) end |
#table ⇒ Object
Delegates to StreamWriter#table. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1661 1662 1663 1664 1665 |
# File 'lib/xlsxrb.rb', line 1661 def table(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.table(...) end |
#validate_data ⇒ Object
Delegates to StreamWriter#validate_data. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1643 1644 1645 1646 1647 |
# File 'lib/xlsxrb.rb', line 1643 def validate_data(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.validate_data(...) end |
#workbook_property ⇒ Object
Delegates to StreamWriter#workbook_property. : (*String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil, **String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil) { (?) -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil } -> String | Integer | bool | Hash[Symbol, String | Integer | bool | nil] | nil
1706 1707 1708 1709 1710 |
# File 'lib/xlsxrb.rb', line 1706 def workbook_property(...) raise Error, "Sheet '' is no longer active. In streaming mode, you cannot write to a previous sheet." if @writer.current_sheet != @sheet_name @writer.workbook_property(...) end |