Class: Xlsxrb::WorksheetBuilder
- Inherits:
-
Object
- Object
- Xlsxrb::WorksheetBuilder
- Defined in:
- lib/xlsxrb.rb,
sig/generated/xlsxrb.rbs
Overview
DSL context for a single worksheet in Xlsxrb.build.
Instance Attribute Summary collapse
-
#styles ⇒ Object
readonly
Internal: returns styles for later processing by WorkbookBuilder : untyped.
Instance Method Summary collapse
-
#auto_filter(range) ⇒ void
Set an auto filter range (e.g. "A1:D10").
-
#build ⇒ Object
: () -> untyped.
-
#chart(**options) {|builder| ... } ⇒ void
Add a chart to the sheet.
-
#column(index, width: nil, hidden: false, custom_width: false, outline_level: nil) ⇒ void
Add a column or multiple columns to the sheet.
-
#comment(cell, text, author: "Author") ⇒ void
Add a comment on a cell.
-
#conditional_format(sqref, **opts) ⇒ void
Add a conditional formatting rule.
-
#filter_column(col_id, filter) ⇒ void
Add a filter column to the auto filter.
-
#freeze_pane(row: 0, col: 0) ⇒ void
Freeze panes at the given row and column.
-
#header_footer(**opts) ⇒ void
Set header/footer text.
-
#hyperlink(cell, url = nil, display: nil, tooltip: nil, location: nil) ⇒ void
Add a hyperlink on a cell.
-
#image(file_data, ext: "png", from_col: 0, from_row: 0, to_col: 5, to_row: 10, **opts) ⇒ void
Insert an image from raw file data.
-
#initialize(name, strict_excel_mode: true) ⇒ WorksheetBuilder
constructor
: (untyped name, ?strict_excel_mode: bool) -> void.
-
#merge(range = nil, row: nil, col_start: nil, col_end: nil, row_start: nil, row_end: nil) ⇒ void
Merge a range of cells (e.g. "A1:B2"), or by coordinate indices.
-
#page_break_col(col_index) ⇒ void
Add a page break before a column.
-
#page_break_row(row_num) ⇒ void
Add a page break before a row.
-
#page_margins(left: nil, right: nil, top: nil, bottom: nil, header: nil, footer: nil) ⇒ void
Set page margins (in inches).
-
#page_setup(**opts) ⇒ void
Set page setup properties.
-
#pivot_table(source_ref, row_fields:, data_fields:, col_fields: [], dest_ref: "E1", name: nil, field_names: nil, items: nil) ⇒ void
Add a pivot table to the sheet.
-
#print_options(name, value) ⇒ void
Set a print option.
-
#protect_sheet(**opts) ⇒ void
Set sheet protection options.
-
#row(values, styles: nil, height: nil, hidden: false, custom_height: false, outline_level: nil) ⇒ void
Add a row to the sheet.
-
#select_cell(active_cell, sqref: nil, pane: nil) ⇒ void
Set active cell selection.
-
#shape(preset: "rect", text: nil, from_col: 0, from_row: 0, to_col: 5, to_row: 5, **opts) ⇒ void
Add a shape to the sheet.
-
#sheet_properties(name, value) ⇒ void
Set a sheet-level property (e.g. :tab_color).
-
#sheet_view(name, value) ⇒ void
Set a sheet view property (e.g. :show_grid_lines, :zoom_scale).
-
#sort_state(ref, sort_conditions, **opts) ⇒ void
Set sort state.
-
#sparkline_group(sparklines:, type: nil, **opts) ⇒ void
Add a sparkline group to the sheet.
-
#split_pane(x_split: 0, y_split: 0, top_left_cell: nil) ⇒ void
Split panes (non-frozen).
-
#style(name, **opts) {|style_builder| ... } ⇒ StyleBuilder
Define a named style that can be applied to cells.
-
#table(ref, columns:, name: nil, display_name: nil, style: nil, **opts) ⇒ void
Add a table to the sheet.
-
#validate_data(sqref, **opts) ⇒ void
Add a data validation rule.
Constructor Details
#initialize(name, strict_excel_mode: true) ⇒ WorksheetBuilder
: (untyped name, ?strict_excel_mode: bool) -> void
873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 |
# File 'lib/xlsxrb.rb', line 873 def initialize(name, strict_excel_mode: true) @name = name @strict_excel_mode = strict_excel_mode @rows = [] @columns = [] @charts = [] @styles = {} # { style_name => StyleBuilder } @style_index_map = {} # { style_name => xf_index } (populated at build time) @hyperlinks = [] @auto_filter = nil @filter_columns = {} @sort_state = nil @data_validations = [] @conditional_formats = [] @tables = [] @comments = [] @sparkline_groups = [] @merge_cells_ranges = [] @freeze_pane = nil @split_pane = nil @selection = nil @page_margins = nil @page_setup = {} @header_footer = {} @print_options = {} @sheet_protection = nil @images = [] @shapes = [] @sheet_properties = {} @sheet_view = {} @row_breaks = [] @col_breaks = [] end |
Instance Attribute Details
#styles ⇒ Object (readonly)
Internal: returns styles for later processing by WorkbookBuilder : untyped
1519 1520 1521 |
# File 'lib/xlsxrb.rb', line 1519 def styles @styles end |
Instance Method Details
#auto_filter(range) ⇒ void
This method returns an undefined value.
Set an auto filter range (e.g. "A1:D10").
: (untyped range) -> untyped
1146 1147 1148 |
# File 'lib/xlsxrb.rb', line 1146 def auto_filter(range) @auto_filter = range end |
#build ⇒ Object
: () -> untyped
1483 1484 1485 1486 1487 1488 1489 1490 1491 1492 1493 1494 1495 1496 1497 1498 1499 1500 1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 1513 1514 1515 |
# File 'lib/xlsxrb.rb', line 1483 def build = {} [:hyperlinks] = @hyperlinks unless @hyperlinks.empty? [:auto_filter] = @auto_filter if @auto_filter [:filter_columns] = @filter_columns unless @filter_columns.empty? [:sort_state] = @sort_state if @sort_state [:data_validations] = @data_validations unless @data_validations.empty? [:conditional_formats] = @conditional_formats unless @conditional_formats.empty? [:tables] = @tables unless @tables.empty? [:pivot_tables] = @pivot_tables unless (@pivot_tables || []).empty? [:comments] = @comments unless @comments.empty? [:sparkline_groups] = @sparkline_groups unless @sparkline_groups.empty? [:merge_cells] = @merge_cells_ranges unless @merge_cells_ranges.empty? [:freeze_pane] = @freeze_pane if @freeze_pane [:split_pane] = @split_pane if @split_pane [:selection] = @selection if @selection [:page_margins] = @page_margins if @page_margins [:page_setup] = @page_setup unless @page_setup.empty? [:header_footer] = @header_footer unless @header_footer.empty? [:print_options] = @print_options unless @print_options.empty? [:sheet_protection] = @sheet_protection if @sheet_protection [:images] = @images unless @images.empty? [:shapes] = @shapes unless @shapes.empty? [:sheet_properties] = @sheet_properties unless @sheet_properties.empty? [:sheet_view] = @sheet_view unless @sheet_view.empty? [:row_breaks] = @row_breaks unless @row_breaks.empty? [:col_breaks] = @col_breaks unless @col_breaks.empty? Elements::Worksheet.new( name: @name, rows: @rows, columns: @columns, charts: @charts, unmapped_data: .empty? ? {} : { facade: } ) end |
#chart(**options) {|builder| ... } ⇒ void
This method returns an undefined value.
Add a chart to the sheet.
: (**untyped options) ?{ (ChartBuilder) -> void } -> void
1108 1109 1110 1111 1112 1113 1114 1115 |
# File 'lib/xlsxrb.rb', line 1108 def chart(**) if block_given? builder = ChartBuilder.new yield builder = builder..merge() end @charts << end |
#column(index, width: nil, hidden: false, custom_width: false, outline_level: nil) ⇒ void
Excel's column width max is 255.
This method returns an undefined value.
Add a column or multiple columns to the sheet.
: (Integer | String | Range[Integer | String] | Array[Integer | String] index, ?width: Float | Integer | nil, ?hidden: bool, ?custom_width: bool, ?outline_level: Integer | nil) -> void
1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 |
# File 'lib/xlsxrb.rb', line 1079 def column(index, width: nil, hidden: false, custom_width: false, outline_level: nil) raise ArgumentError, "Column width #{width} must be between 0 and 255 characters (Excel limitation)" if @strict_excel_mode && width && (width.negative? || width > 255) indices = case index when Range, Array index.map { |i| Elements::Cell.column_index(i) } else [Elements::Cell.column_index(index)] end indices.each do |idx| @columns << Elements::Column.new( index: idx, width: width, hidden: hidden, custom_width: custom_width || !width.nil?, outline_level: outline_level ) end end |
#comment(cell, text, author: "Author") ⇒ void
This method returns an undefined value.
Add a comment on a cell.
: (String | Integer cell, String text, ?author: ::String) -> void
1256 1257 1258 |
# File 'lib/xlsxrb.rb', line 1256 def comment(cell, text, author: "Author") @comments << { cell: cell, text: text, author: } end |
#conditional_format(sqref, **opts) ⇒ void
This method returns an undefined value.
Add a conditional formatting rule.
: (untyped sqref, **untyped opts) -> untyped
1195 1196 1197 |
# File 'lib/xlsxrb.rb', line 1195 def conditional_format(sqref, **opts) @conditional_formats << opts.merge(sqref: sqref) end |
#filter_column(col_id, filter) ⇒ void
This method returns an undefined value.
Add a filter column to the auto filter.
: (untyped col_id, untyped filter) -> untyped
1157 1158 1159 |
# File 'lib/xlsxrb.rb', line 1157 def filter_column(col_id, filter) @filter_columns[col_id] = filter end |
#freeze_pane(row: 0, col: 0) ⇒ void
This method returns an undefined value.
Freeze panes at the given row and column.
: (?row: Integer, ?col: (Integer | String)) -> void
1325 1326 1327 1328 |
# File 'lib/xlsxrb.rb', line 1325 def freeze_pane(row: 0, col: 0) col = Elements::Cell.column_index(col) @freeze_pane = { row: row, col: col } end |
#header_footer(**opts) ⇒ void
This method returns an undefined value.
Set header/footer text.
: (**untyped opts) -> void
1388 1389 1390 |
# File 'lib/xlsxrb.rb', line 1388 def (**opts) @header_footer.merge!(opts) end |
#hyperlink(cell, url = nil, display: nil, tooltip: nil, location: nil) ⇒ void
This method returns an undefined value.
Add a hyperlink on a cell.
: (String | Integer cell, ?String? url, ?display: String?, ?tooltip: String?, ?location: String?) -> void
1129 1130 1131 1132 1133 1134 1135 1136 |
# File 'lib/xlsxrb.rb', line 1129 def hyperlink(cell, url = nil, display: nil, tooltip: nil, location: nil) link = { cell: cell } link[:url] = url if url link[:display] = display if display link[:tooltip] = tooltip if tooltip link[:location] = location if location @hyperlinks << link end |
#image(file_data, ext: "png", from_col: 0, from_row: 0, to_col: 5, to_row: 10, **opts) ⇒ void
This method returns an undefined value.
Insert an image from raw file data. : (String file_data, ?ext: ::String, ?from_col: ::Integer, ?from_row: ::Integer, ?to_col: ::Integer, ?to_row: ::Integer, **untyped opts) -> void
1430 1431 1432 1433 1434 |
# File 'lib/xlsxrb.rb', line 1430 def image(file_data, ext: "png", from_col: 0, from_row: 0, to_col: 5, to_row: 10, **opts) img = { file_data: file_data, ext: ext, from_col: from_col, from_row: from_row, to_col: to_col, to_row: to_row } img.merge!(opts) @images << img end |
#merge(range = nil, row: nil, col_start: nil, col_end: nil, row_start: nil, row_end: nil) ⇒ void
This method returns an undefined value.
Merge a range of cells (e.g. "A1:B2"), or by coordinate indices.
: (?(String | Hash[Symbol, Integer | String])? range, ?row: Integer?, ?col_start: (Integer | String)?, ?col_end: (Integer | String)?, ?row_start: Integer?, ?row_end: Integer?) -> void
1290 1291 1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 |
# File 'lib/xlsxrb.rb', line 1290 def merge(range = nil, row: nil, col_start: nil, col_end: nil, row_start: nil, row_end: nil) if range.is_a?(Hash) row = range[:row] row_start = range[:row_start] row_end = range[:row_end] col_start = range[:col_start] col_end = range[:col_end] range = nil end if range raise ArgumentError, "Invalid merge range format: '#{range}'. Expected format like 'A1:B2'." if @strict_excel_mode && !range.match?(/^[A-Za-z]{1,3}\d+(:[A-Za-z]{1,3}\d+)?$/) return if @merge_cells_ranges.include?(range) @merge_cells_ranges << range else r_start = row || row_start || 0 r_end = row || row_end || 0 c_start = Elements::Cell.column_index(col_start || 0) c_end = Elements::Cell.column_index(col_end || 0) start_ref = "#{Xlsxrb::Elements::Cell.column_letter(c_start)}#{r_start + 1}" end_ref = "#{Xlsxrb::Elements::Cell.column_letter(c_end)}#{r_end + 1}" @merge_cells_ranges << "#{start_ref}:#{end_ref}" end end |
#page_break_col(col_index) ⇒ void
This method returns an undefined value.
Add a page break before a column. : (Integer | String col_index) -> void
1476 1477 1478 1479 |
# File 'lib/xlsxrb.rb', line 1476 def page_break_col(col_index) col_index = Elements::Cell.column_index(col_index) @col_breaks << col_index end |
#page_break_row(row_num) ⇒ void
This method returns an undefined value.
Add a page break before a row. : (Integer row_num) -> void
1469 1470 1471 |
# File 'lib/xlsxrb.rb', line 1469 def page_break_row(row_num) @row_breaks << row_num end |
#page_margins(left: nil, right: nil, top: nil, bottom: nil, header: nil, footer: nil) ⇒ void
This method returns an undefined value.
Set page margins (in inches).
: (?left: Float?, ?right: Float?, ?top: Float?, ?bottom: Float?, ?header: Float?, ?footer: Float?) -> void
1368 1369 1370 |
# File 'lib/xlsxrb.rb', line 1368 def page_margins(left: nil, right: nil, top: nil, bottom: nil, header: nil, footer: nil) @page_margins = { left: left, right: right, top: top, bottom: bottom, header: header, footer: }.compact end |
#page_setup(**opts) ⇒ void
This method returns an undefined value.
Set page setup properties.
: (**untyped opts) -> void
1378 1379 1380 |
# File 'lib/xlsxrb.rb', line 1378 def page_setup(**opts) @page_setup.merge!(opts) end |
#pivot_table(source_ref, row_fields:, data_fields:, col_fields: [], dest_ref: "E1", name: nil, field_names: nil, items: nil) ⇒ void
This method returns an undefined value.
Add a pivot table to the sheet.
: (untyped source_ref, **untyped opts) -> void
1236 1237 1238 1239 1240 1241 1242 1243 1244 |
# File 'lib/xlsxrb.rb', line 1236 def pivot_table(source_ref, row_fields:, data_fields:, col_fields: [], dest_ref: "E1", name: nil, field_names: nil, items: nil) @pivot_tables ||= [] @pivot_tables << { source_ref: source_ref, row_fields: row_fields, data_fields: data_fields, col_fields: col_fields, dest_ref: dest_ref, name: name, field_names: field_names, items: items } end |
#print_options(name, value) ⇒ void
This method returns an undefined value.
Set a print option.
: (Symbol name, untyped value) -> void
1399 1400 1401 |
# File 'lib/xlsxrb.rb', line 1399 def (name, value) @print_options[name] = value end |
#protect_sheet(**opts) ⇒ void
This method returns an undefined value.
Set sheet protection options.
: (**untyped opts) -> void
1411 1412 1413 1414 1415 1416 1417 1418 1419 1420 1421 1422 1423 |
# File 'lib/xlsxrb.rb', line 1411 def protect_sheet(**opts) normalized = opts.dup plain_password = normalized[:password] needs_hash = plain_password.is_a?(String) && !plain_password.empty? && normalized[:algorithm_name].nil? && normalized[:hash_value].nil? && normalized[:salt_value].nil? && normalized[:spin_count].nil? && !plain_password.match?(/\A[0-9A-Fa-f]{4}\z/) if needs_hash normalized.delete(:password) normalized.merge!(Xlsxrb::Ooxml::Utils.hash_password(plain_password)) end @sheet_protection = normalized end |
#row(values, styles: nil, height: nil, hidden: false, custom_height: false, outline_level: nil) ⇒ void
Excel's column limit is 16,384, row limit is 1,048,576, string max length is 32,767.
This method returns an undefined value.
Add a row to the sheet.
: (Array | Hash[untyped, untyped] values, ?styles: untyped, ?height: Float | Integer | nil, ?hidden: bool, ?custom_height: bool, ?outline_level: Integer | nil) -> void
936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 1065 1066 |
# File 'lib/xlsxrb.rb', line 936 def row(values, styles: nil, height: nil, hidden: false, custom_height: false, outline_level: nil) row_index = @rows.size # See: https://support.microsoft.com/en-us/office/excel-specifications-and-limits-1672b34d-7043-467e-8e27-269d656771c3 if @strict_excel_mode raise ArgumentError, "Row index #{row_index} exceeds Excel limit of 1,048,576 rows" if row_index >= 1_048_576 raise ArgumentError, "Row height #{height} must be between 0 and 409 points (Excel limitation)" if height && (height.negative? || height > 409) end if values.is_a?(Hash) max_col = values.keys.map { |k| Elements::Cell.column_index(k) }.max || -1 cells_array = Array.new(max_col + 1) values.each do |k, v| idx = Elements::Cell.column_index(k) cells_array[idx] = v end values = cells_array end if styles.is_a?(Hash) = {} styles.each do |k, v| if k.is_a?(Range) || k.is_a?(Array) k.each { |idx| [Elements::Cell.column_index(idx)] = v } else [Elements::Cell.column_index(k)] = v end end max_col_style = .keys.max || -1 styles_array = Array.new(max_col_style + 1) .each do |idx, v| styles_array[idx] = v end styles = styles_array end # Auto-detect Date / Time for built-in styles values.each_with_index do |val, idx| cell_style = styles.is_a?(Array) ? styles[idx] : styles if val.is_a?(Date) && cell_style.nil? style("__xlsxrb_date", number_format: "yyyy-mm-dd") unless @styles.key?("__xlsxrb_date") styles = [] if styles.nil? styles = Array.new(values.size, styles) unless styles.is_a?(Array) styles[idx] = "__xlsxrb_date" elsif val.is_a?(Time) && cell_style.nil? style("__xlsxrb_time", number_format: "yyyy-mm-dd hh:mm:ss") unless @styles.key?("__xlsxrb_time") styles = [] if styles.nil? styles = Array.new(values.size, styles) unless styles.is_a?(Array) styles[idx] = "__xlsxrb_time" end end max_len = values.size max_len = [max_len, styles.size].max if styles.is_a?(Array) # See: https://support.microsoft.com/en-us/office/excel-specifications-and-limits-1672b34d-7043-467e-8e27-269d656771c3 raise ArgumentError, "Row contains #{max_len} columns, exceeding Excel limit of 16_384 columns" if @strict_excel_mode && max_len > 16_384 cells = Array.new(max_len) style_lookup = styles.is_a?(Array) col_index = 0 while col_index < max_len val = col_index < values.size ? values[col_index] : nil raise ArgumentError, "Invalid cell value type or value: #{val.class} for value #{val.inspect}" unless val.nil? || val.is_a?(String) || (val.is_a?(Numeric) && !(val.is_a?(Float) && (val.infinite? || val.nan?))) || val.is_a?(TrueClass) || val.is_a?(FalseClass) || val.is_a?(Date) || val.is_a?(Time) || val.is_a?(Elements::Formula) || (val.is_a?(Hash) && val.key?(:formula)) || val.is_a?(Elements::RichText) || (val.is_a?(Array) && val.first.is_a?(Hash) && (val.first.key?(:text) || val.first.key?("text"))) # See: https://support.microsoft.com/en-us/office/excel-specifications-and-limits-1672b34d-7043-467e-8e27-269d656771c3 raise ArgumentError, "Cell text length #{val.length} exceeds Excel limit of 32,767 characters" if @strict_excel_mode && val.is_a?(String) && val.length > 32_767 if val.is_a?(Array) && val.first.is_a?(Hash) && (val.first.key?(:text) || val.first.key?("text")) # Coerce array of hashes to RichText runs = val.map do |run| text = run[:text] || run["text"] font = run.reject { |k| k.to_s == "text" } { text: text, font: font.empty? ? nil : font }.compact end val = Elements::RichText.new(runs: runs) end style_name = if style_lookup col_index < styles.size ? styles[col_index] : nil else styles end if style_name.is_a?(Hash) inline_name = "__inline_#{style_name.hash}" style(inline_name, **style_name) unless @styles.key?(inline_name) style_name = inline_name end if val.nil? && style_name.nil? col_index += 1 next end # If value is a Formula object or Hash with :formula, store it as the cell's formula cells[col_index] = if val.is_a?(Elements::Formula) Elements::Cell.new( row_index: row_index, column_index: col_index, value: nil, formula: val, style_index: style_name ) elsif val.is_a?(Hash) && val.key?(:formula) f_obj = Elements::Formula.new(val[:formula]) Elements::Cell.new( row_index: row_index, column_index: col_index, value: val[:value], formula: f_obj, style_index: style_name ) else Elements::Cell.new( row_index: row_index, column_index: col_index, value: val, style_index: style_name ) end col_index += 1 end cells.compact! @rows << Elements::Row.new( index: row_index, cells: cells, height: height, hidden: hidden, custom_height: custom_height || !height.nil?, outline_level: outline_level ) end |
#select_cell(active_cell, sqref: nil, pane: nil) ⇒ void
This method returns an undefined value.
Set active cell selection.
: (String active_cell, ?sqref: String?, ?pane: (String | Symbol)?) -> void
1350 1351 1352 1353 |
# File 'lib/xlsxrb.rb', line 1350 def select_cell(active_cell, sqref: nil, pane: nil) @selection = { active_cell: active_cell, sqref: sqref || active_cell } @selection[:pane] = pane if pane end |
#shape(preset: "rect", text: nil, from_col: 0, from_row: 0, to_col: 5, to_row: 5, **opts) ⇒ void
This method returns an undefined value.
Add a shape to the sheet. : (**untyped opts) -> void
1441 1442 1443 1444 1445 1446 |
# File 'lib/xlsxrb.rb', line 1441 def shape(preset: "rect", text: nil, from_col: 0, from_row: 0, to_col: 5, to_row: 5, **opts) shape = { preset: preset, text: text, from_col: from_col, from_row: from_row, to_col: to_col, to_row: to_row } shape[:name] = opts.delete(:name) || "Shape #{@shapes.size + 1}" shape.merge!(opts) @shapes << shape end |
#sheet_properties(name, value) ⇒ void
This method returns an undefined value.
Set a sheet-level property (e.g. :tab_color). : (Symbol name, untyped value) -> void
1453 1454 1455 |
# File 'lib/xlsxrb.rb', line 1453 def sheet_properties(name, value) @sheet_properties[name] = value end |
#sheet_view(name, value) ⇒ void
This method returns an undefined value.
Set a sheet view property (e.g. :show_grid_lines, :zoom_scale). : (Symbol name, untyped value) -> void
1460 1461 1462 |
# File 'lib/xlsxrb.rb', line 1460 def sheet_view(name, value) @sheet_view[name] = value end |
#sort_state(ref, sort_conditions, **opts) ⇒ void
This method returns an undefined value.
Set sort state.
: (untyped ref, untyped sort_conditions, **untyped opts) -> untyped
1169 1170 1171 |
# File 'lib/xlsxrb.rb', line 1169 def sort_state(ref, sort_conditions, **opts) @sort_state = { ref: ref, sort_conditions: sort_conditions }.merge(opts) end |
#sparkline_group(sparklines:, type: nil, **opts) ⇒ void
This method returns an undefined value.
Add a sparkline group to the sheet.
: (**untyped opts) -> void
1270 1271 1272 1273 1274 1275 |
# File 'lib/xlsxrb.rb', line 1270 def sparkline_group(sparklines:, type: nil, **opts) group = { sparklines: sparklines } group[:type] = type if type group.merge!(opts) @sparkline_groups << group end |
#split_pane(x_split: 0, y_split: 0, top_left_cell: nil) ⇒ void
This method returns an undefined value.
Split panes (non-frozen).
: (?x_split: ::Integer, ?y_split: ::Integer, ?top_left_cell: String?) -> void
1338 1339 1340 |
# File 'lib/xlsxrb.rb', line 1338 def split_pane(x_split: 0, y_split: 0, top_left_cell: nil) @split_pane = { x_split: x_split, y_split: y_split, top_left_cell: top_left_cell } end |
#style(name, **opts) {|style_builder| ... } ⇒ StyleBuilder
Define a named style that can be applied to cells.
: (String name, **untyped opts) ?{ (StyleBuilder) -> void } -> StyleBuilder
916 917 918 919 920 921 922 |
# File 'lib/xlsxrb.rb', line 916 def style(name, **opts) style_builder = StyleBuilder.new(name) style_builder.(**opts) unless opts.empty? yield style_builder if block_given? @styles[name] = style_builder style_builder end |
#table(ref, columns:, name: nil, display_name: nil, style: nil, **opts) ⇒ void
This method returns an undefined value.
Add a table to the sheet.
: (untyped ref, columns: untyped, ?name: untyped?, ?display_name: untyped?, ?style: untyped?, **untyped opts) -> untyped
1212 1213 1214 1215 1216 1217 1218 1219 |
# File 'lib/xlsxrb.rb', line 1212 def table(ref, columns:, name: nil, display_name: nil, style: nil, **opts) tbl = { ref: ref, columns: columns } tbl[:name] = name if name tbl[:display_name] = display_name if display_name tbl[:style] = style if style tbl.merge!(opts) @tables << tbl end |
#validate_data(sqref, **opts) ⇒ void
This method returns an undefined value.
Add a data validation rule.
: (untyped sqref, **untyped opts) -> untyped
1182 1183 1184 |
# File 'lib/xlsxrb.rb', line 1182 def validate_data(sqref, **opts) @data_validations << opts.merge(sqref: sqref) end |