Class: Xlsxrb::WorksheetBuilder
- Inherits:
-
Object
- Object
- Xlsxrb::WorksheetBuilder
- Defined in:
- lib/xlsxrb.rb
Overview
DSL context for a single worksheet in Xlsxrb.build.
Instance Attribute Summary collapse
-
#styles ⇒ Object
readonly
Internal: returns styles for later processing by WorkbookBuilder : Elements::Workbook.
Instance Method Summary collapse
-
#auto_filter(range) ⇒ void
Set an auto filter range (e.g. "A1:D10").
-
#build ⇒ Object
: () -> Elements::Worksheet.
-
#chart(**options) {|builder| ... } ⇒ void
Add a chart to the sheet.
-
#column(index, width: nil, hidden: false, custom_width: false, outline_level: nil) ⇒ void
Add a column or multiple columns to the sheet.
-
#comment(cell, text, author: "Author") ⇒ void
Add a comment on a cell.
-
#conditional_format(sqref, **opts) ⇒ void
Add a conditional formatting rule.
-
#filter_column(col_id, filter) ⇒ void
Add a filter column to the auto filter.
-
#freeze_pane(row: 0, col: 0) ⇒ void
Freeze panes at the given row and column.
-
#header_footer(**opts) ⇒ void
Set header/footer text.
-
#hyperlink(cell, url = nil, display: nil, tooltip: nil, location: nil) ⇒ void
Add a hyperlink on a cell.
-
#image(file_data, ext: "png", from_col: 0, from_row: 0, to_col: 5, to_row: 10, **opts) ⇒ Object
Insert an image from raw file data.
-
#initialize(name, strict_excel_mode: true) ⇒ WorksheetBuilder
constructor
: (untyped name, ?strict_excel_mode: bool) -> void.
-
#merge(range = nil, row: nil, col_start: nil, col_end: nil, row_start: nil, row_end: nil) ⇒ void
Merge a range of cells (e.g. "A1:B2"), or by coordinate indices.
-
#page_break_col(col_index) ⇒ Object
Add a page break before a column.
-
#page_break_row(row_num) ⇒ Object
Add a page break before a row.
-
#page_margins(left: nil, right: nil, top: nil, bottom: nil, header: nil, footer: nil) ⇒ void
Set page margins (in inches).
-
#page_setup(**opts) ⇒ void
Set page setup properties.
-
#pivot_table(source_ref, row_fields:, data_fields:, col_fields: [], dest_ref: "E1", name: nil, field_names: nil, items: nil) ⇒ void
Add a pivot table to the sheet.
-
#print_options(name, value) ⇒ void
Set a print option.
-
#protect_sheet(**opts) ⇒ void
Set sheet protection options.
-
#row(values, styles: nil, height: nil, hidden: false, custom_height: false, outline_level: nil) ⇒ void
Add a row to the sheet.
-
#select_cell(active_cell, sqref: nil, pane: nil) ⇒ void
Set active cell selection.
-
#shape(preset: "rect", text: nil, from_col: 0, from_row: 0, to_col: 5, to_row: 5, **opts) ⇒ Object
Add a shape to the sheet.
-
#sheet_properties(name, value) ⇒ Object
Set a sheet-level property (e.g. :tab_color).
-
#sheet_view(name, value) ⇒ Object
Set a sheet view property (e.g. :show_grid_lines, :zoom_scale).
-
#sort_state(ref, sort_conditions, **opts) ⇒ void
Set sort state.
-
#sparkline_group(sparklines:, type: nil, **opts) ⇒ void
Add a sparkline group to the sheet.
-
#split_pane(x_split: 0, y_split: 0, top_left_cell: nil) ⇒ void
Split panes (non-frozen).
-
#style(name, **opts) {|style_builder| ... } ⇒ StyleBuilder
Define a named style that can be applied to cells.
-
#table(ref, columns:, name: nil, display_name: nil, style: nil, **opts) ⇒ void
Add a table to the sheet.
-
#validate_data(sqref, **opts) ⇒ void
Add a data validation rule.
Constructor Details
#initialize(name, strict_excel_mode: true) ⇒ WorksheetBuilder
: (untyped name, ?strict_excel_mode: bool) -> void
807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 |
# File 'lib/xlsxrb.rb', line 807 def initialize(name, strict_excel_mode: true) @name = name @strict_excel_mode = strict_excel_mode @rows = [] @columns = [] @charts = [] @styles = {} # { style_name => StyleBuilder } @style_index_map = {} # { style_name => xf_index } (populated at build time) @hyperlinks = [] @auto_filter = nil @filter_columns = {} @sort_state = nil @data_validations = [] @conditional_formats = [] @tables = [] @comments = [] @sparkline_groups = [] @merge_cells_ranges = [] @freeze_pane = nil @split_pane = nil @selection = nil @page_margins = nil @page_setup = {} @header_footer = {} @print_options = {} @sheet_protection = nil @images = [] @shapes = [] @sheet_properties = {} @sheet_view = {} @row_breaks = [] @col_breaks = [] end |
Instance Attribute Details
#styles ⇒ Object (readonly)
Internal: returns styles for later processing by WorkbookBuilder : Elements::Workbook
1444 1445 1446 |
# File 'lib/xlsxrb.rb', line 1444 def styles @styles end |
Instance Method Details
#auto_filter(range) ⇒ void
This method returns an undefined value.
Set an auto filter range (e.g. "A1:D10").
: (untyped range) -> untyped
1080 1081 1082 |
# File 'lib/xlsxrb.rb', line 1080 def auto_filter(range) @auto_filter = range end |
#build ⇒ Object
: () -> Elements::Worksheet
1408 1409 1410 1411 1412 1413 1414 1415 1416 1417 1418 1419 1420 1421 1422 1423 1424 1425 1426 1427 1428 1429 1430 1431 1432 1433 1434 1435 1436 1437 1438 1439 1440 |
# File 'lib/xlsxrb.rb', line 1408 def build = {} [:hyperlinks] = @hyperlinks unless @hyperlinks.empty? [:auto_filter] = @auto_filter if @auto_filter [:filter_columns] = @filter_columns unless @filter_columns.empty? [:sort_state] = @sort_state if @sort_state [:data_validations] = @data_validations unless @data_validations.empty? [:conditional_formats] = @conditional_formats unless @conditional_formats.empty? [:tables] = @tables unless @tables.empty? [:pivot_tables] = @pivot_tables unless (@pivot_tables || []).empty? [:comments] = @comments unless @comments.empty? [:sparkline_groups] = @sparkline_groups unless @sparkline_groups.empty? [:merge_cells] = @merge_cells_ranges unless @merge_cells_ranges.empty? [:freeze_pane] = @freeze_pane if @freeze_pane [:split_pane] = @split_pane if @split_pane [:selection] = @selection if @selection [:page_margins] = @page_margins if @page_margins [:page_setup] = @page_setup unless @page_setup.empty? [:header_footer] = @header_footer unless @header_footer.empty? [:print_options] = @print_options unless @print_options.empty? [:sheet_protection] = @sheet_protection if @sheet_protection [:images] = @images unless @images.empty? [:shapes] = @shapes unless @shapes.empty? [:sheet_properties] = @sheet_properties unless @sheet_properties.empty? [:sheet_view] = @sheet_view unless @sheet_view.empty? [:row_breaks] = @row_breaks unless @row_breaks.empty? [:col_breaks] = @col_breaks unless @col_breaks.empty? Elements::Worksheet.new( name: @name, rows: @rows, columns: @columns, charts: @charts, unmapped_data: .empty? ? {} : { facade: } ) end |
#chart(**options) {|builder| ... } ⇒ void
This method returns an undefined value.
Add a chart to the sheet.
: (**String | Integer | bool | nil options) ?{ (ChartBuilder) -> void } -> void
1042 1043 1044 1045 1046 1047 1048 1049 |
# File 'lib/xlsxrb.rb', line 1042 def chart(**) if block_given? builder = ChartBuilder.new yield builder = builder..merge() end @charts << end |
#column(index, width: nil, hidden: false, custom_width: false, outline_level: nil) ⇒ void
Excel's column width max is 255.
This method returns an undefined value.
Add a column or multiple columns to the sheet.
: (Integer | String | Range[Integer | String] | Array[Integer | String] index, ?width: Float | Integer | nil, ?hidden: bool, ?custom_width: bool, ?outline_level: Integer | nil) -> void
1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 |
# File 'lib/xlsxrb.rb', line 1013 def column(index, width: nil, hidden: false, custom_width: false, outline_level: nil) raise ArgumentError, "Column width #{width} must be between 0 and 255 characters (Excel limitation)" if @strict_excel_mode && width && (width.negative? || width > 255) indices = case index when Range, Array index.map { |i| Elements::Cell.column_index(i) } else [Elements::Cell.column_index(index)] end indices.each do |idx| @columns << Elements::Column.new( index: idx, width: width, hidden: hidden, custom_width: custom_width || !width.nil?, outline_level: outline_level ) end end |
#comment(cell, text, author: "Author") ⇒ void
This method returns an undefined value.
Add a comment on a cell.
: (String | Integer cell, String text, ?author: ::String) -> void
1190 1191 1192 |
# File 'lib/xlsxrb.rb', line 1190 def comment(cell, text, author: "Author") @comments << { cell: cell, text: text, author: } end |
#conditional_format(sqref, **opts) ⇒ void
This method returns an undefined value.
Add a conditional formatting rule.
: (untyped sqref, **untyped opts) -> untyped
1129 1130 1131 |
# File 'lib/xlsxrb.rb', line 1129 def conditional_format(sqref, **opts) @conditional_formats << opts.merge(sqref: sqref) end |
#filter_column(col_id, filter) ⇒ void
This method returns an undefined value.
Add a filter column to the auto filter.
: (untyped col_id, untyped filter) -> untyped
1091 1092 1093 |
# File 'lib/xlsxrb.rb', line 1091 def filter_column(col_id, filter) @filter_columns[col_id] = filter end |
#freeze_pane(row: 0, col: 0) ⇒ void
This method returns an undefined value.
Freeze panes at the given row and column.
: (?row: ::Integer, ?col: ::Integer) -> void
1250 1251 1252 1253 |
# File 'lib/xlsxrb.rb', line 1250 def freeze_pane(row: 0, col: 0) col = Elements::Cell.column_index(col) @freeze_pane = { row: row, col: col } end |
#header_footer(**opts) ⇒ void
This method returns an undefined value.
Set header/footer text.
: (**String | Integer | bool | nil opts) -> void
1313 1314 1315 |
# File 'lib/xlsxrb.rb', line 1313 def (**opts) @header_footer.merge!(opts) end |
#hyperlink(cell, url = nil, display: nil, tooltip: nil, location: nil) ⇒ void
This method returns an undefined value.
Add a hyperlink on a cell.
: (String | Integer cell, ?String? url, ?display: String?, ?tooltip: String?, ?location: String?) -> void
1063 1064 1065 1066 1067 1068 1069 1070 |
# File 'lib/xlsxrb.rb', line 1063 def hyperlink(cell, url = nil, display: nil, tooltip: nil, location: nil) link = { cell: cell } link[:url] = url if url link[:display] = display if display link[:tooltip] = tooltip if tooltip link[:location] = location if location @hyperlinks << link end |
#image(file_data, ext: "png", from_col: 0, from_row: 0, to_col: 5, to_row: 10, **opts) ⇒ Object
Insert an image from raw file data. : (String file_data, ?ext: ::String, ?from_col: ::Integer, ?from_row: ::Integer, ?to_col: ::Integer, ?to_row: ::Integer, **String | Integer | bool | nil opts) -> void
1355 1356 1357 1358 1359 |
# File 'lib/xlsxrb.rb', line 1355 def image(file_data, ext: "png", from_col: 0, from_row: 0, to_col: 5, to_row: 10, **opts) img = { file_data: file_data, ext: ext, from_col: from_col, from_row: from_row, to_col: to_col, to_row: to_row } img.merge!(opts) @images << img end |
#merge(range = nil, row: nil, col_start: nil, col_end: nil, row_start: nil, row_end: nil) ⇒ void
This method returns an undefined value.
Merge a range of cells (e.g. "A1:B2"), or by coordinate indices.
: (?String? range, ?String? row, ?String? col_start, ?String? col_end, ?String? row_start, ?String? row_end) -> void
1224 1225 1226 1227 1228 1229 1230 1231 1232 1233 1234 1235 1236 1237 1238 1239 |
# File 'lib/xlsxrb.rb', line 1224 def merge(range = nil, row: nil, col_start: nil, col_end: nil, row_start: nil, row_end: nil) if range raise ArgumentError, "Invalid merge range format: '#{range}'. Expected format like 'A1:B2'." if @strict_excel_mode && !range.match?(/^[A-Za-z]{1,3}\d+(:[A-Za-z]{1,3}\d+)?$/) return if @merge_cells_ranges.include?(range) @merge_cells_ranges << range else r_start = row || row_start || 0 r_end = row || row_end || 0 c_start = col_start || 0 c_end = col_end || 0 start_ref = "#{Xlsxrb::Elements::Cell.column_letter(c_start)}#{r_start + 1}" end_ref = "#{Xlsxrb::Elements::Cell.column_letter(c_end)}#{r_end + 1}" @merge_cells_ranges << "#{start_ref}:#{end_ref}" end end |
#page_break_col(col_index) ⇒ Object
Add a page break before a column. : (Integer col_index) -> void
1401 1402 1403 1404 |
# File 'lib/xlsxrb.rb', line 1401 def page_break_col(col_index) col_index = Elements::Cell.column_index(col_index) @col_breaks << col_index end |
#page_break_row(row_num) ⇒ Object
Add a page break before a row. : (Integer row_num) -> void
1394 1395 1396 |
# File 'lib/xlsxrb.rb', line 1394 def page_break_row(row_num) @row_breaks << row_num end |
#page_margins(left: nil, right: nil, top: nil, bottom: nil, header: nil, footer: nil) ⇒ void
This method returns an undefined value.
Set page margins (in inches).
: (?left: Float?, ?right: Float?, ?top: Float?, ?bottom: Float?, ?header: Float?, ?footer: Float?) -> void
1293 1294 1295 |
# File 'lib/xlsxrb.rb', line 1293 def page_margins(left: nil, right: nil, top: nil, bottom: nil, header: nil, footer: nil) @page_margins = { left: left, right: right, top: top, bottom: bottom, header: header, footer: }.compact end |
#page_setup(**opts) ⇒ void
This method returns an undefined value.
Set page setup properties.
: (**String | Integer | bool | nil opts) -> void
1303 1304 1305 |
# File 'lib/xlsxrb.rb', line 1303 def page_setup(**opts) @page_setup.merge!(opts) end |
#pivot_table(source_ref, row_fields:, data_fields:, col_fields: [], dest_ref: "E1", name: nil, field_names: nil, items: nil) ⇒ void
1170 1171 1172 1173 1174 1175 1176 1177 1178 |
# File 'lib/xlsxrb.rb', line 1170 def pivot_table(source_ref, row_fields:, data_fields:, col_fields: [], dest_ref: "E1", name: nil, field_names: nil, items: nil) @pivot_tables ||= [] @pivot_tables << { source_ref: source_ref, row_fields: row_fields, data_fields: data_fields, col_fields: col_fields, dest_ref: dest_ref, name: name, field_names: field_names, items: items } end |
#print_options(name, value) ⇒ void
This method returns an undefined value.
Set a print option.
: (Symbol name, String | Integer | bool value) -> void
1324 1325 1326 |
# File 'lib/xlsxrb.rb', line 1324 def (name, value) @print_options[name] = value end |
#protect_sheet(**opts) ⇒ void
This method returns an undefined value.
Set sheet protection options.
: (**String | Integer | bool | nil opts) -> void
1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 1348 |
# File 'lib/xlsxrb.rb', line 1336 def protect_sheet(**opts) normalized = opts.dup plain_password = normalized[:password] needs_hash = plain_password.is_a?(String) && !plain_password.empty? && normalized[:algorithm_name].nil? && normalized[:hash_value].nil? && normalized[:salt_value].nil? && normalized[:spin_count].nil? && !plain_password.match?(/\A[0-9A-Fa-f]{4}\z/) if needs_hash normalized.delete(:password) normalized.merge!(Xlsxrb::Ooxml::Utils.hash_password(plain_password)) end @sheet_protection = normalized end |
#row(values, styles: nil, height: nil, hidden: false, custom_height: false, outline_level: nil) ⇒ void
Excel's column limit is 16,384, row limit is 1,048,576, string max length is 32,767.
This method returns an undefined value.
Add a row to the sheet.
: (Array[String | Numeric | bool | nil] | Hash[Integer | String, String | Numeric | bool | nil] values, ?styles: String | Hash[Symbol, String | Integer | bool | nil] | Array[String | Hash[Symbol, String | Integer | bool | nil] | nil] | nil, ?height: Float | Integer | nil, ?hidden: bool, ?custom_height: bool, ?outline_level: Integer | nil) -> void
870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 |
# File 'lib/xlsxrb.rb', line 870 def row(values, styles: nil, height: nil, hidden: false, custom_height: false, outline_level: nil) row_index = @rows.size # See: https://support.microsoft.com/en-us/office/excel-specifications-and-limits-1672b34d-7043-467e-8e27-269d656771c3 if @strict_excel_mode raise ArgumentError, "Row index #{row_index} exceeds Excel limit of 1,048,576 rows" if row_index >= 1_048_576 raise ArgumentError, "Row height #{height} must be between 0 and 409 points (Excel limitation)" if height && (height.negative? || height > 409) end if values.is_a?(Hash) max_col = values.keys.map { |k| Elements::Cell.column_index(k) }.max || -1 cells_array = Array.new(max_col + 1) values.each do |k, v| idx = Elements::Cell.column_index(k) cells_array[idx] = v end values = cells_array end if styles.is_a?(Hash) = {} styles.each do |k, v| if k.is_a?(Range) || k.is_a?(Array) k.each { |idx| [Elements::Cell.column_index(idx)] = v } else [Elements::Cell.column_index(k)] = v end end max_col_style = .keys.max || -1 styles_array = Array.new(max_col_style + 1) .each do |idx, v| styles_array[idx] = v end styles = styles_array end # Auto-detect Date / Time for built-in styles values.each_with_index do |val, idx| cell_style = styles.is_a?(Array) ? styles[idx] : styles if val.is_a?(Date) && cell_style.nil? style("__xlsxrb_date", number_format: "yyyy-mm-dd") unless @styles.key?("__xlsxrb_date") styles = [] if styles.nil? styles = Array.new(values.size, styles) unless styles.is_a?(Array) styles[idx] = "__xlsxrb_date" elsif val.is_a?(Time) && cell_style.nil? style("__xlsxrb_time", number_format: "yyyy-mm-dd hh:mm:ss") unless @styles.key?("__xlsxrb_time") styles = [] if styles.nil? styles = Array.new(values.size, styles) unless styles.is_a?(Array) styles[idx] = "__xlsxrb_time" end end max_len = values.size max_len = [max_len, styles.size].max if styles.is_a?(Array) # See: https://support.microsoft.com/en-us/office/excel-specifications-and-limits-1672b34d-7043-467e-8e27-269d656771c3 raise ArgumentError, "Row contains #{max_len} columns, exceeding Excel limit of 16_384 columns" if @strict_excel_mode && max_len > 16_384 cells = Array.new(max_len) style_lookup = styles.is_a?(Array) col_index = 0 while col_index < max_len val = col_index < values.size ? values[col_index] : nil raise ArgumentError, "Invalid cell value type or value: #{val.class} for value #{val.inspect}" unless val.nil? || val.is_a?(String) || (val.is_a?(Numeric) && !(val.is_a?(Float) && (val.infinite? || val.nan?))) || val.is_a?(TrueClass) || val.is_a?(FalseClass) || val.is_a?(Date) || val.is_a?(Time) || val.is_a?(Elements::Formula) || (val.is_a?(Hash) && val.key?(:formula)) || val.is_a?(Elements::RichText) || (val.is_a?(Array) && val.first.is_a?(Hash) && (val.first.key?(:text) || val.first.key?("text"))) # See: https://support.microsoft.com/en-us/office/excel-specifications-and-limits-1672b34d-7043-467e-8e27-269d656771c3 raise ArgumentError, "Cell text length #{val.length} exceeds Excel limit of 32,767 characters" if @strict_excel_mode && val.is_a?(String) && val.length > 32_767 if val.is_a?(Array) && val.first.is_a?(Hash) && (val.first.key?(:text) || val.first.key?("text")) # Coerce array of hashes to RichText runs = val.map do |run| text = run[:text] || run["text"] font = run.reject { |k| k.to_s == "text" } { text: text, font: font.empty? ? nil : font }.compact end val = Elements::RichText.new(runs: runs) end style_name = if style_lookup col_index < styles.size ? styles[col_index] : nil else styles end if style_name.is_a?(Hash) inline_name = "__inline_#{style_name.hash}" style(inline_name, **style_name) unless @styles.key?(inline_name) style_name = inline_name end if val.nil? && style_name.nil? col_index += 1 next end # If value is a Formula object or Hash with :formula, store it as the cell's formula cells[col_index] = if val.is_a?(Elements::Formula) Elements::Cell.new( row_index: row_index, column_index: col_index, value: nil, formula: val, style_index: style_name ) elsif val.is_a?(Hash) && val.key?(:formula) f_obj = Elements::Formula.new(val[:formula]) Elements::Cell.new( row_index: row_index, column_index: col_index, value: val[:value], formula: f_obj, style_index: style_name ) else Elements::Cell.new( row_index: row_index, column_index: col_index, value: val, style_index: style_name ) end col_index += 1 end cells.compact! @rows << Elements::Row.new( index: row_index, cells: cells, height: height, hidden: hidden, custom_height: custom_height || !height.nil?, outline_level: outline_level ) end |
#select_cell(active_cell, sqref: nil, pane: nil) ⇒ void
This method returns an undefined value.
Set active cell selection.
: (String active_cell, ?sqref: String?, ?pane: String?) -> void
1275 1276 1277 1278 |
# File 'lib/xlsxrb.rb', line 1275 def select_cell(active_cell, sqref: nil, pane: nil) @selection = { active_cell: active_cell, sqref: sqref || active_cell } @selection[:pane] = pane if pane end |
#shape(preset: "rect", text: nil, from_col: 0, from_row: 0, to_col: 5, to_row: 5, **opts) ⇒ Object
Add a shape to the sheet. : (?preset: ::String, ?text: String?, ?from_col: ::Integer, ?from_row: ::Integer, ?to_col: ::Integer, ?to_row: ::Integer, **String | Integer | bool | nil opts) -> void
1366 1367 1368 1369 1370 1371 |
# File 'lib/xlsxrb.rb', line 1366 def shape(preset: "rect", text: nil, from_col: 0, from_row: 0, to_col: 5, to_row: 5, **opts) shape = { preset: preset, text: text, from_col: from_col, from_row: from_row, to_col: to_col, to_row: to_row } shape[:name] = opts.delete(:name) || "Shape #{@shapes.size + 1}" shape.merge!(opts) @shapes << shape end |
#sheet_properties(name, value) ⇒ Object
Set a sheet-level property (e.g. :tab_color). : (Symbol name, String | Integer | bool | Float value) -> void
1378 1379 1380 |
# File 'lib/xlsxrb.rb', line 1378 def sheet_properties(name, value) @sheet_properties[name] = value end |
#sheet_view(name, value) ⇒ Object
Set a sheet view property (e.g. :show_grid_lines, :zoom_scale). : (Symbol name, String | Integer | bool | Float value) -> void
1385 1386 1387 |
# File 'lib/xlsxrb.rb', line 1385 def sheet_view(name, value) @sheet_view[name] = value end |
#sort_state(ref, sort_conditions, **opts) ⇒ void
This method returns an undefined value.
Set sort state.
: (untyped ref, untyped sort_conditions, **untyped opts) -> untyped
1103 1104 1105 |
# File 'lib/xlsxrb.rb', line 1103 def sort_state(ref, sort_conditions, **opts) @sort_state = { ref: ref, sort_conditions: sort_conditions }.merge(opts) end |
#sparkline_group(sparklines:, type: nil, **opts) ⇒ void
This method returns an undefined value.
Add a sparkline group to the sheet.
: (sparklines: Array, ?type: String?, **String | Integer | bool | nil opts) -> void
1204 1205 1206 1207 1208 1209 |
# File 'lib/xlsxrb.rb', line 1204 def sparkline_group(sparklines:, type: nil, **opts) group = { sparklines: sparklines } group[:type] = type if type group.merge!(opts) @sparkline_groups << group end |
#split_pane(x_split: 0, y_split: 0, top_left_cell: nil) ⇒ void
This method returns an undefined value.
Split panes (non-frozen).
: (?x_split: ::Integer, ?y_split: ::Integer, ?top_left_cell: String?) -> void
1263 1264 1265 |
# File 'lib/xlsxrb.rb', line 1263 def split_pane(x_split: 0, y_split: 0, top_left_cell: nil) @split_pane = { x_split: x_split, y_split: y_split, top_left_cell: top_left_cell } end |
#style(name, **opts) {|style_builder| ... } ⇒ StyleBuilder
Define a named style that can be applied to cells.
: (String name, **String | Integer | bool | nil opts) ?{ (WorksheetBuilder) -> void } -> void
850 851 852 853 854 855 856 |
# File 'lib/xlsxrb.rb', line 850 def style(name, **opts) style_builder = StyleBuilder.new(name) style_builder.(**opts) unless opts.empty? yield style_builder if block_given? @styles[name] = style_builder style_builder end |
#table(ref, columns:, name: nil, display_name: nil, style: nil, **opts) ⇒ void
This method returns an undefined value.
Add a table to the sheet.
: (untyped ref, columns: untyped, ?name: untyped?, ?display_name: untyped?, ?style: untyped?, **untyped opts) -> untyped
1146 1147 1148 1149 1150 1151 1152 1153 |
# File 'lib/xlsxrb.rb', line 1146 def table(ref, columns:, name: nil, display_name: nil, style: nil, **opts) tbl = { ref: ref, columns: columns } tbl[:name] = name if name tbl[:display_name] = display_name if display_name tbl[:style] = style if style tbl.merge!(opts) @tables << tbl end |
#validate_data(sqref, **opts) ⇒ void
This method returns an undefined value.
Add a data validation rule.
: (untyped sqref, **untyped opts) -> untyped
1116 1117 1118 |
# File 'lib/xlsxrb.rb', line 1116 def validate_data(sqref, **opts) @data_validations << opts.merge(sqref: sqref) end |